\version "2.24.0"

\header {
  texidoc = "
Sometimes a parenthesized note should include its accidental sign
and/or dot. Using @code{\\parenthesize} won't do that.

The defined function @code{\\parentheAll} will.  

"
  doctitle = "Including accidental and dots in a parenthesized note"
}

%%%% http://lsr.di.unimi.it/LSR/Item?id=972
% added by PPS on Jan. 2015.
%% => https://code.google.com/p/lilypond/issues/detail?id=155
%% => http://lilypond.1069038.n5.nabble.com/Pitched-trill-glissando-tc170876.html
%%
%% The functionality of this snippet is default for 2.19.20 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LSR workaround:
#(set! paper-alist (cons '("snippet" . (cons (* 210 mm) (* 50 mm))) paper-alist))
\paper {
  #(set-paper-size "snippet")
  ragged-right = ##f
  tagline = ##f
}
\markup\vspace #1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

\paper {
  indent = 0
}

parentheAll = #(define-music-function (note) (ly:music?)
#{
  \once \override Parentheses.font-size = #-1
  \once \override Parentheses.stencil = #(lambda (grob)
       (let* ((acc (ly:grob-object (ly:grob-parent grob Y) 'accidental-grob))
              (dot (ly:grob-object (ly:grob-parent grob Y) 'dot)))
         (if (not (null? acc)) (ly:pointer-group-interface::add-grob grob 'elements acc))
         (if (not (null? dot)) (ly:pointer-group-interface::add-grob grob 'elements dot))
         (parentheses-interface::print grob)))
  \parenthesize $note
#})

\score {
   {
    \mark\markup\small\column { 
      \italic "Using:" \vspace #.3
      \typewriter "\parenthesize" 
    }
    \parenthesize cis''4.
    s1
    \stopStaff
    s1
    \startStaff
    s4 \bar ""
    \mark\markup\small\column { 
      \italic "Using:" \vspace #.3
      \typewriter "\parentheAll" 
    }
    \parentheAll cis''!4.
    s1 s4
    \stopStaff
    s1
    \startStaff
    s4 \bar ""
    \tweak extra-offset #'(1 . 1.5)
    \mark\markup\small\column { 
      \italic "e.g. when applied in a" \vspace #.3
      \typewriter "\pitchedTrill"  \vspace #.2
      \italic "context:"
    }
  \pitchedTrill b2\startTrillSpan 
  -\tweak minimum-length #10
  -\tweak springs-and-rods #ly:spanner::set-spacing-rods
  -\tweak bound-details.right.Y #-2.7
  -\tweak bound-details.left.padding #3.5
  -\tweak bound-details.right.padding #.8
  \glissando ~ ces'
  \once \omit Staff.Flag
  \once \omit Staff.Stem
  \once \override NoteHead.font-size = #-4
  \once \override NoteHead.X-offset = #-.4
  \grace {  \parentheAll c'!8\stopTrillSpan }
  b8 r8 
  }
  \layout {
    \context {
      \Score
      \omit TimeSignature
      \override Clef.transparent = ##t
      \override BarLine.transparent = ##t
      \override RehearsalMark.self-alignment-X = #LEFT
    }
  }
}

